Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

USE work.pkg_081006.all;

----------------------------------------------------
-- This module provides timing and control for the
-- Booster BLM Integrator control.

entity Integrator_Sequencer is
port(
		clk			: in std_logic;
		nreset		: in std_logic;
		Reset_Seq	: in std_logic;
--		ADC_Done	: in std_logic;
		
		Intg_CLK	: out std_logic;
		INIT_DAQ	: out std_logic;
		ADC_CNVST	: out std_logic;
		hold_B_12, sel_B_12n, reset_B_12n : out std_logic;
		hold_B_34, sel_B_34n, reset_B_34n : out std_logic;
		hold_A_12, sel_A_12n, reset_A_12n : out std_logic;
		hold_A_34, sel_A_34n, reset_A_34n : out std_logic
		                             
		
);
end Integrator_Sequencer;

--########################################################
--				A R C H I T E C T U R E
--########################################################
architecture main of Integrator_Sequencer is

-- Synchronized Input Signals --------------------------------
	signal rst_seq		: std_logic;
--	signal adc_rdy		: std_logic;

-- Integrator 20us Clock Timer -------------------------------
	signal timer		: std_logic_vector(11 downto 0);
	signal rst_timer	: std_logic;

-- Sequencer Output Signals ----------------------------------
	signal sel_1		: std_logic;
	signal sel_2		: std_logic;
	signal hld_1		: std_logic;
	signal hld_2		: std_logic;
	signal rst_1		: std_logic;
	signal rst_2		: std_logic;
	
	signal sample		: std_logic;
	signal initial		: std_logic;
		
-- SETUP_STATE_MACHINE ---------------------------------------
	type STATE_TYPE is
	(init, I1_hold, I1_conv, I1_reset, I1_wait, 
	 I2_hold, I2_conv, I2_reset, I2_wait);
	signal CS, NS : STATE_TYPE;

--========================================================
--						B E G I N
--========================================================
begin

SYNC_RESET: led 
	port map(sig_in=>Reset_Seq, clk=>clk,	clrn=>nreset, pls_out=>rst_seq); 

--SYNC_ADC: led 
--	port map(sig_in=>ADC_Done, clk=>clk,	clrn=>nreset, pls_out=>adc_rdy); 

----------------------------------------------------------
INTEGRATOR_20us_CLK:
process(clk,nreset,timer, rst_seq)
begin
	if(nreset='0' OR rst_seq='1') then 
		timer <= (others => '0');
	elsif(rising_edge(clk))	then 
		if 	 (timer < X"1F3") then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;

		if 	 (timer < X"0F9") then 	Intg_CLK <= '1';
		else						Intg_CLK <= '0';
		end if;
	end if;
end process;
		
--{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{
--                S T A T E   M A C H I N E
--}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}
SM_REG: process(clk, nreset, rst_seq)
begin
	if(nreset='0' OR rst_seq='1')	then CS <= init;
	elsif(rising_edge(clk))	 		then CS <= NS;
	end if;
end process;

SM_NEXT_STATE: 
process(CS, timer)
begin
	sel_1 		<= '0';
	sel_2 		<= '0';
	hld_1 		<= '0';
	hld_2 		<= '0';
	rst_1 		<= '0';
	rst_2 		<= '0';
	sample		<= '0';
	initial		<= '0';

	case CS is
		when init =>			
			initial		<= '1';--Hold off any ADC Strobes during initialization
			rst_1 		<= '1';
			rst_2 		<= '1';
			if(timer = X"1F3")then 		NS <= I1_hold; -- timer = 500 cnts = 20.00us
			else                   	    NS <= init;
			end if;
		
		---------------------------------------------------------------------------
		-- Integrate on Integrator B while Integrator A is Digitized and Reset	
		when I1_hold =>					
			hld_1		<= '1';
			sel_1		<= '1';
			if(timer = X"020")then 		NS <= I1_conv; -- timer = 32 cnts = 1.28us
			else                   	    NS <= I1_hold;
			end if;

		when I1_conv =>					
			hld_1		<= '1';
			sel_1		<= '1';
			sample		<= '1';
			if(timer = X"070")then 		NS <= I1_reset; -- timer = 112 cnts = 4.48us
			else                   	    NS <= I1_conv;
			end if;

		when I1_reset =>					
			hld_1		<= '1';
			rst_1		<= '1';
			sel_2		<= '1';
			if(timer = X"1E0")then 		NS <= I1_wait; -- timer = 480 cnts = 19.20us
			else                   	    NS <= I1_reset;
			end if;

		when I1_wait =>					
			hld_1		<= '1';
			sel_2		<= '1';
			if(timer = X"1F3")then 		NS <= I2_hold; -- timer = 500 cnts = 20.00us
			else                   	    NS <= I1_wait;
			end if;

		-----------------------------------------------------------------------------
		-- Integrate on Integrator A while Integrator B is Digitized and Reset	
		when I2_hold =>					
			hld_2		<= '1';
			sel_2		<= '1';
			if(timer = X"020")then 		NS <= I2_conv; -- timer = 32 cnts = 1.28us
			else                   	    NS <= I2_hold;
			end if;

		when I2_conv =>					
			hld_2		<= '1';
			sel_2		<= '1';
			sample		<= '1';
			if(timer = X"070")then 		NS <= I2_reset; -- timer = 112 cnts = 4.48us
			else                   	    NS <= I2_conv;
			end if;

		when I2_reset =>					
			hld_2		<= '1';
			rst_2		<= '1';
			sel_1		<= '1';
			if(timer = X"1E0")then 		NS <= I2_wait; -- timer = 480 cnts = 19.20us
			else                   	    NS <= I2_reset;
			end if;

		when I2_wait =>					
			hld_2		<= '1';
			sel_1		<= '1';
			if(timer = X"1F3")then 		NS <= I1_hold; -- timer = 500 cnts = 20.00us
			else                   	    NS <= I2_wait;
			end if;

		when others => 					NS <= init;
	end case;
end process;

process(clk, nreset)
begin
	if(rising_edge(clk))	then 
		ADC_CNVST	<= sample;
		INIT_DAQ	<= initial; --Hold off any ADC Strobes during initialization	
		
		hold_B_12	<= hld_2; 
		hold_B_34	<= hld_2;
		hold_A_12	<= hld_1; 
		hold_A_34	<= hld_1; 

		sel_B_12n	<= not sel_2; 
		sel_B_34n	<= not sel_2;  
		sel_A_12n	<= not sel_1;  
		sel_A_34n	<= not sel_1;  

		reset_B_12n	<= not rst_2;
		reset_B_34n	<= not rst_2;
		reset_A_12n	<= not rst_1;
		reset_A_34n	<= not rst_1;
	end if;
end process;	
	
end;             

